if(0)
          {
                  asm("NOP");
                  FLASH_EraseOptionByte(0x4810);
                  asm("NOP");
                  kk = FLASH_GetCodeSize();
                  asm("NOP");
                  FLASH_EraseBlock(0x07,FLASH_MemType_Data);
                  asm("NOP");asm("NOP");
                  asm("NOP");
          }
      {
            //data_deal();
            FLASH_Unlock(FLASH_MemType_Data);
            FLASH_Unlock(FLASH_MemType_Program);
            FLASH_ProgramByte(0x1000,0xff);
            FLASH_ProgramWord(0x1110,0xdddddddd);
            FLASH_ProgramWord(0x1080,0xdddddddd);
            FLASH_ProgramWord(0x10f0,0xdddddddd);
            FLASH_ProgramWord(0x13f0,0xdddddddd);
            kk = FLASH_ReadByte(0x1000);
            FLASH_ProgramOptionByte(0x4810,0x11);
      }